@@ -112,8 +112,7 @@ type Resolver struct {
 	// Characters to drop from the user part.
 	DropChars string
 
-	// Path to resolve and exist hooks.
-	ExistsHook  string
+	// Path to the resolve hook.
 	ResolveHook string
 
 	// Map of domain -> alias files for that domain.
@@ -140,49 +139,69 @@ func NewResolver() *Resolver {
 // Resolve the given address, returning the list of corresponding recipients
 // (if any).
 func (v *Resolver) Resolve(addr string) ([]Recipient, error) {
-	return v.resolve(0, addr)
+	tr := trace.New("Alias.Resolve", addr)
+	defer tr.Finish()
+	return v.resolve(0, addr, tr)
 }
 
 // Exists check that the address exists in the database.
 // It returns the cleaned address, and a boolean indicating the result.
 // The clean address can be used to look it up in other databases, even if it
-// doesn't exist.
+// doesn't exist. It must only be called for local addresses.
 func (v *Resolver) Exists(addr string) (string, bool) {
-	v.mu.Lock()
+	tr := trace.New("Alias.Exists", addr)
+	defer tr.Finish()
+
 	addr = v.cleanIfLocal(addr)
-	_, ok := v.aliases[addr]
-	v.mu.Unlock()
 
-	if ok {
-		return addr, true
-	}
-
-	return addr, v.runExistsHook(addr)
+	rcpts, _ := v.lookup(addr, tr)
+	return addr, len(rcpts) > 0
 }
 
-func (v *Resolver) resolve(rcount int, addr string) ([]Recipient, error) {
+func (v *Resolver) lookup(addr string, tr *trace.Trace) ([]Recipient, error) {
+	v.mu.Lock()
+	rcpts := v.aliases[addr]
+	v.mu.Unlock()
+
+	// Augment with the hook results.
+	hr, err := v.runResolveHook(addr)
+	if err != nil {
+		tr.Debugf("lookup(%q) hook error: %v", addr, err)
+		return nil, err
+	}
+
+	tr.Debugf("lookup(%q) -> %v + %v", addr, rcpts, hr)
+	return append(rcpts, hr...), nil
+}
+
+func (v *Resolver) resolve(rcount int, addr string, tr *trace.Trace) ([]Recipient, error) {
+	tr.Debugf("%d| resolve(%d, %q)", rcount, rcount, addr)
 	if rcount >= recursionLimit {
 		return nil, ErrRecursionLimitExceeded
 	}
 
+	// If the address is not local, we return it as-is, so delivery is
+	// attempted against it.
+	// Example: an alias that resolves to a non-local address.
+	if _, ok := v.domains[envelope.DomainOf(addr)]; !ok {
+		tr.Debugf("%d| non-local domain, returning %q", rcount, addr)
+		return []Recipient{{addr, EMAIL}}, nil
+	}
+
 	// Drop suffixes and chars to get the "clean" address before resolving.
 	// This also means that we will return the clean version if there's no
 	// match, which our callers can rely upon.
 	addr = v.cleanIfLocal(addr)
 
 	// Lookup in the aliases database.
-	v.mu.Lock()
-	rcpts := v.aliases[addr]
-	v.mu.Unlock()
-
-	// Augment with the hook results.
-	hr, err := v.runResolveHook(addr)
+	rcpts, err := v.lookup(addr, tr)
 	if err != nil {
+		tr.Debugf("%d| error in lookup: %v", rcount, err)
 		return nil, err
 	}
-	rcpts = append(rcpts, hr...)
 
 	if len(rcpts) == 0 {
+		tr.Debugf("%d| no aliases found, returning %q", rcount, addr)
 		return []Recipient{{addr, EMAIL}}, nil
 	}
 
@@ -194,14 +213,16 @@ func (v *Resolver) resolve(rcount int, addr string) ([]Recipient, error) {
 			continue
 		}
 
-		ar, err := v.resolve(rcount+1, r.Addr)
+		ar, err := v.resolve(rcount+1, r.Addr, tr)
 		if err != nil {
+			tr.Debugf("%d| resolve(%q) returned error: %v", rcount, r.Addr, err)
 			return nil, err
 		}
 
 		ret = append(ret, ar...)
 	}
 
+	tr.Debugf("%d| returning %v", rcount, ret)
 	return ret, nil
 }
 
@@ -435,35 +456,3 @@ func (v *Resolver) runResolveHook(addr string) ([]Recipient, error) {
 	hookResults.Add("resolve:success", 1)
 	return rs, nil
 }
-
-func (v *Resolver) runExistsHook(addr string) bool {
-	if v.ExistsHook == "" {
-		hookResults.Add("exists:notset", 1)
-		return false
-	}
-	// TODO: check if the file is executable.
-	if _, err := os.Stat(v.ExistsHook); os.IsNotExist(err) {
-		hookResults.Add("exists:skip", 1)
-		return false
-	}
-
-	// TODO: this should be done via a context propagated all the way through.
-	tr := trace.New("Hook.Alias-Exists", addr)
-	defer tr.Finish()
-
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	cmd := exec.CommandContext(ctx, v.ExistsHook, addr)
-
-	out, err := cmd.CombinedOutput()
-	tr.Debugf("output: %q", string(out))
-	if err != nil {
-		tr.Debugf("not exists: %v", err)
-		hookResults.Add("exists:false", 1)
-		return false
-	}
-
-	tr.Debugf("exists")
-	hookResults.Add("exists:true", 1)
-	return true
-}
